"""
Скрипт для получения данных о сетевых адаптерах
используемых в системе, а также их параметров, которые
доступны при выводе команды "ipconfig /all".
"""

import json
import subprocess
from ipaddress import IPv4Address, IPv6Address
from pathlib import Path
from socket import socket, AF_INET, SOCK_DGRAM


def all_adapters():
    """
    Парсинг вывода команды ipconfig /all.
    Выполняем команду. Для того чтобы разделить вывод на блоки,
    в каждом из которых будет один адаптер, приводим строку к нужному
    виду, указав в качестве разделителя "~". Также на будущее делим все
    остальные строки с помощью "||".
    Затем итерируемся по созданному нами текстовому блоку. Делим его по
    разделителям для начала на блоки, в которых содержаться данные об
    одном адаптере. Затем, в каждом блоке делим содержимое на список из
    строк по "||". Затем итерируемся по полученному списку, забираем
    название адаптера. Создаем в словаре раздел с адаптером. Итерируемся
    дальше, пропускаем пустые строки, проверяем адреса ipv4 и ipv6, для
    того, чтобы убрать у них приставку (в русском варианте: Основной).
    Делим элементы списка и добавляем в словарь полученные значения.
    Затем записываем полученные данные в json для лучшей сохранности и
    возвращаем полученный словарь из функции.
    :return: словарь со всеми адаптерами в системе.
    """

    network_adapters = dict()

    out = subprocess.check_output("ipconfig /all", shell=True).decode("cp866")
    text = ""

    for item in out.split("\r\n"):
        if item == "":
            continue
        elif "Ethernet" in item:
            text = f"{text.strip()}~{item.strip()}"
        else:
            text = f"{text.strip()}||{item.strip()}"

    for tx in text.split("~")[1:]:
        for item in tx.split("||"):
            if "Ethernet" in item:
                network_adapters[item.replace(":", "")] = dict()
                keys = network_adapters[item.replace(":", "")]
                continue
            try:
                if item.split(" :")[1].strip() == "":
                    continue
            except IndexError:
                continue
            try:
                IPv4Address(item.split(" :")[1].strip().split("(")[0])
                keys.update({item.split(" :")[0].replace(".", "").strip(): item.split(" :")[1].strip().split("(")[0]})
                continue
            except Exception:
                pass
            try:
                IPv6Address(item.split(" :")[1].strip().split("(")[0])
                keys.update({item.split(" :")[0].replace(".", "").strip(): item.split(" :")[1].strip().split("(")[0]})
                continue
            except Exception:
                keys.update({item.split(" :")[0].replace(".", "").strip(): item.split(" :")[1].strip()})

    with open(Path.cwd() / 'ethernet_adapters.json', 'w', encoding='utf-8') as js:
        json.dump(network_adapters, js, indent=4, ensure_ascii=False)

    return network_adapters


def local_ipv4() -> str:
    """
    Получаем локальный IP-адрес с помощью установки
    соединения на адрес 10.255.255.255. В ответ получаем
    имя сокета, которое и является адресом.

    :return: возвращает локальный IP-адрес или адрес локальной петли
    в случае возникновения исключения.
    """
    ip = None
    st = socket(AF_INET, SOCK_DGRAM)
    try:
        st.connect(('10.255.255.255', 1))
        ip = st.getsockname()[0]
    except Exception:
        ip = '127.0.0.1'
    finally:
        st.close()
        return ip


def default_network(network_adapters: dict, loc_ip: str) -> dict:
    """
    Получаем сетевой адаптер используемый по-умолчанию.
    Итерируемся по ранее составленному словарю, составляем промежуточный
    словарь, итерируемся по нему и проверяем, есть ли в нем значение,
    которое соответствует ipv4-адресу по-умолчанию. Если да, возвращаем
    словарь из функции. Если нет, обнуляем словарь и двигаемся дальше.
    :param network_adapters: словарь со всеми сетевыми адаптерами в системе.
    :param loc_ip: локальный ipv4 адрес.
    :return: словарь с сетевым адаптером по-умолчанию.
    """
    default_adapter = dict()
    for network in network_adapters:
        default_adapter[network] = dict()
        for adapter in network_adapters[network]:
            default_adapter[network].update({adapter: network_adapters[network][adapter]})
        for loc in default_adapter[network]:
            if loc_ip == default_adapter[network][loc]:
                with open(Path.cwd() / 'ethernet_default.json', 'w', encoding='utf-8') as js:
                    json.dump(default_adapter, js, indent=4, ensure_ascii=False)
                return default_adapter
        default_adapter.clear()


def main():
    adapters = all_adapters()
    ipv4 = local_ipv4()
    default = default_network(adapters, ipv4)
    if default is None:
        if adapters is not None:
            print(adapters)
    else:
        print(adapters)
        print(default)


if __name__ == "__main__":
    main()
